"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.definitions = void 0;
const fz = __importStar(require("../converters/fromZigbee"));
const tz = __importStar(require("../converters/toZigbee"));
const constants = __importStar(require("../lib/constants"));
const exposes = __importStar(require("../lib/exposes"));
const legacy = __importStar(require("../lib/legacy"));
const m = __importStar(require("../lib/modernExtend"));
const reporting = __importStar(require("../lib/reporting"));
const utils = __importStar(require("../lib/utils"));
const e = exposes.presets;
const acova = {
    fz: {
        thermostat: {
            ...fz.thermostat,
            convert: (model, msg, publish, options, meta) => {
                const result = fz.thermostat.convert(model, msg, publish, options, meta);
                if (result && msg.data.systemMode !== undefined) {
                    result.system_mode = utils.getFromLookup(msg.data.systemMode, constants.acovaThermostatSystemModes);
                }
                if (model.model === "ALCANTARA2") {
                    result.local_temperature = null;
                }
                return result;
            },
        },
    },
    tz: {
        acova_thermostat_system_mode: {
            key: ["system_mode"],
            convertSet: async (entity, key, value, meta) => {
                let systemMode = utils.getKey(constants.acovaThermostatSystemModes, value, undefined, Number);
                if (systemMode === undefined) {
                    systemMode = utils.getKey(legacy.thermostatSystemModes, value, value, Number);
                }
                await entity.write("hvacThermostat", { systemMode });
                return { state: { system_mode: value } };
            },
            convertGet: async (entity, key, meta) => {
                await entity.read("hvacThermostat", ["systemMode"]);
            },
        },
    },
};
exports.definitions = [
    {
        zigbeeModel: [
            "ALCANTARA2 D1.00P1.01Z1.00\u0000\u0000\u0000\u0000\u0000\u0000",
            "ALCANTARA2 D1.00P1.02Z1.00\u0000\u0000\u0000\u0000\u0000\u0000",
        ],
        model: "ALCANTARA2",
        vendor: "Acova",
        description: "Alcantara 2 heater",
        fromZigbee: [acova.fz.thermostat, fz.hvac_user_interface],
        toZigbee: [
            tz.thermostat_local_temperature,
            acova.tz.acova_thermostat_system_mode,
            tz.thermostat_occupied_heating_setpoint,
            tz.thermostat_unoccupied_heating_setpoint,
            tz.thermostat_running_state,
        ],
        exposes: [
            e
                .climate()
                .withSetpoint("occupied_heating_setpoint", 7, 28, 0.5)
                .withSetpoint("unoccupied_heating_setpoint", 7, 28, 0.5)
                .withLocalTemperature()
                .withSystemMode(["off", "heat", "auto"])
                .withRunningState(["idle", "heat"]),
        ],
        configure: async (device, coordinatorEndpoint) => {
            const endpoint = device.getEndpoint(1);
            await reporting.bind(endpoint, coordinatorEndpoint, ["genPowerCfg", "hvacThermostat"]);
            await reporting.thermostatTemperature(endpoint);
            await reporting.thermostatRunningState(endpoint);
            await reporting.thermostatOccupiedHeatingSetpoint(endpoint);
            await reporting.thermostatUnoccupiedHeatingSetpoint(endpoint);
        },
    },
    {
        zigbeeModel: ["ALCANTARA3"],
        model: "ALCANTARA3",
        vendor: "Acova",
        description: "Alcantara 3 heater",
        fromZigbee: [acova.fz.thermostat, fz.hvac_user_interface, fz.electrical_measurement],
        toZigbee: [
            tz.thermostat_local_temperature,
            acova.tz.acova_thermostat_system_mode,
            tz.thermostat_occupied_heating_setpoint,
            tz.thermostat_unoccupied_heating_setpoint,
            tz.thermostat_running_state,
        ],
        exposes: [
            e
                .climate()
                .withSetpoint("occupied_heating_setpoint", 7, 28, 0.5)
                .withSetpoint("unoccupied_heating_setpoint", 7, 28, 0.5)
                .withLocalTemperature()
                .withSystemMode(["off", "heat", "auto"])
                .withRunningState(["idle", "heat"]),
            e.power_apparent(),
        ],
        configure: async (device, coordinatorEndpoint) => {
            const endpoint = device.getEndpoint(1);
            await reporting.bind(endpoint, coordinatorEndpoint, ["genPowerCfg", "hvacThermostat", "haElectricalMeasurement"]);
            await reporting.thermostatTemperature(endpoint);
            await reporting.thermostatRunningState(endpoint);
            await reporting.thermostatOccupiedHeatingSetpoint(endpoint);
            await reporting.thermostatUnoccupiedHeatingSetpoint(endpoint);
            await reporting.apparentPower(endpoint);
        },
    },
    {
        zigbeeModel: [
            "PERCALE2 D1.00P1.01Z1.00",
            "PERCALE2 D1.00P1.02Z1.00",
            "PERCALE2 D1.00P1.03Z1.00",
            "TAFFETAS2 D1.00P1.01Z1.00\u0000\u0000\u0000\u0000\u0000\u0000\u0000",
            "TAFFETAS2 D1.00P1.02Z1.00\u0000\u0000\u0000\u0000\u0000\u0000\u0000",
            "TAFFETAS2 D1.00P1.03Z1.00",
            "TAFFETAS2 D1.00P1.63Z1.19",
        ],
        model: "TAFFETAS2/PERCALE2",
        vendor: "Acova",
        description: "Taffetas 2 / Percale 2 heater",
        fromZigbee: [fz.thermostat, fz.hvac_user_interface, fz.occupancy],
        toZigbee: [
            tz.thermostat_local_temperature,
            acova.tz.acova_thermostat_system_mode,
            tz.thermostat_occupied_heating_setpoint,
            tz.thermostat_unoccupied_heating_setpoint,
            tz.thermostat_running_state,
            tz.thermostat_local_temperature_calibration,
        ],
        exposes: [
            e
                .climate()
                .withSetpoint("occupied_heating_setpoint", 7, 28, 0.5)
                .withSetpoint("unoccupied_heating_setpoint", 7, 28, 0.5)
                .withLocalTemperature()
                .withSystemMode(["off", "heat", "auto"])
                .withRunningState(["idle", "heat"])
                .withLocalTemperatureCalibration(),
            e.occupancy(),
        ],
        configure: async (device, coordinatorEndpoint) => {
            const endpoint = device.getEndpoint(1);
            const endpoint2 = device.getEndpoint(2);
            await reporting.bind(endpoint, coordinatorEndpoint, ["genPowerCfg", "hvacThermostat"]);
            await reporting.bind(endpoint2, coordinatorEndpoint, ["msOccupancySensing"]);
            await reporting.thermostatTemperature(endpoint);
            await reporting.thermostatRunningState(endpoint);
            await reporting.thermostatOccupiedHeatingSetpoint(endpoint);
            await reporting.thermostatUnoccupiedHeatingSetpoint(endpoint);
            await reporting.thermostatTemperatureCalibration(endpoint);
            await reporting.occupancy(endpoint2);
        },
    },
    {
        zigbeeModel: ["IHC-Enki"],
        model: "IHC-Enki",
        vendor: "Acova",
        description: "Acova Madras IHC towel radiator (Zigbee thermostat)",
        extend: [
            m.thermostat({
                setpoints: { values: { occupiedHeatingSetpoint: { min: 7, max: 30, step: 0.5 } } },
                localTemperatureCalibration: { values: { min: -5, max: 5, step: 0.1 } },
                systemMode: { values: ["off", "heat", "auto"] },
            }),
        ],
    },
];
//# sourceMappingURL=acova.js.map