"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZnpVersion = void 0;
var ZnpVersion;
(function (ZnpVersion) {
    ZnpVersion[ZnpVersion["ZStack12"] = 0] = "ZStack12";
    ZnpVersion[ZnpVersion["ZStack3x0"] = 1] = "ZStack3x0";
    ZnpVersion[ZnpVersion["ZStack30x"] = 2] = "ZStack30x";
})(ZnpVersion || (exports.ZnpVersion = ZnpVersion = {}));
//# sourceMappingURL=tstype.js.map