import{u as a,a as m,I as p,r as c,s as d,j as e,dr as f}from"./index-B_eahVWI.js";import{I as l}from"./InfoAlert-BYb1Ijsc.js";import{S as g}from"./SettingsList-BKLVWCQb.js";import"./envs-CRUy-_iG.js";import"./CheckboxField-CzUa3ISp.js";import"./InputField-DBG-IjF8.js";import"./NumberField-Cft7S9El.js";import"./SelectField-JWPpuidh.js";function k({sourceIdx:t,group:r}){const{t:o}=a(["settings","common"]),i=m(p(s=>s.bridgeInfo[t])),n=c.useCallback(async s=>{await d(t,"bridge/request/group/options",{id:r.id.toString(),options:s})},[t,r.id]);return e.jsxs(e.Fragment,{children:[e.jsx(l,{children:e.jsx("a",{href:f,target:"_blank",rel:"noreferrer",className:"link link-hover",children:o(s=>s.read_the_docs_info,{ns:"common"})})}),e.jsx(g,{schema:i.config_schema.definitions.group??{properties:{}},data:i.config.groups[r.id]??{},set:n,namespace:"definitions-group"})]})}export{k as default};
